/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_PROXY_H
#define JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/visualization/visualizationsocb_common.h"
#include <string>

namespace jac {
namespace ap {
namespace visualization {
namespace proxy {
namespace events {
    using CameraFEH265ImagePDataViz = ara::com::internal::proxy::event::EventAdapter<::CameraH265Image>;
    using LidarInno3PPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInno4PPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    static constexpr ara::com::internal::EntityId VisualizationSoCBCameraFEH265ImagePDataVizId = 7559U; //CameraFEH265ImagePDataViz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationSoCBLidarInno3PPC2VizId = 59449U; //LidarInno3PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationSoCBLidarInno4PPC2VizId = 16882U; //LidarInno4PPC2Viz_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class VisualizationSoCBProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              CameraFEH265ImagePDataViz(proxyAdapter->GetProxy(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarInno3PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationSoCBLidarInno3PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarInno4PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationSoCBLidarInno4PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::CameraFEH265ImagePDataViz GetCameraFEH265ImagePDataViz() noexcept { return std::move(CameraFEH265ImagePDataViz); }
        events::LidarInno3PPC2Viz GetLidarInno3PPC2Viz() noexcept { return std::move(LidarInno3PPC2Viz); }
        events::LidarInno4PPC2Viz GetLidarInno4PPC2Viz() noexcept { return std::move(LidarInno4PPC2Viz); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::CameraFEH265ImagePDataViz CameraFEH265ImagePDataViz;
        events::LidarInno3PPC2Viz LidarInno3PPC2Viz;
        events::LidarInno4PPC2Viz LidarInno4PPC2Viz;
    };

    virtual ~VisualizationSoCBProxy()
    {
        CameraFEH265ImagePDataViz.UnsetReceiveHandler();
        CameraFEH265ImagePDataViz.Unsubscribe();
        LidarInno3PPC2Viz.UnsetReceiveHandler();
        LidarInno3PPC2Viz.Unsubscribe();
        LidarInno4PPC2Viz.UnsetReceiveHandler();
        LidarInno4PPC2Viz.Unsubscribe();

    }

    explicit VisualizationSoCBProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, handle)),
          CameraFEH265ImagePDataViz(proxyAdapter->GetProxy(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarInno3PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationSoCBLidarInno3PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarInno4PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationSoCBLidarInno4PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    VisualizationSoCBProxy(const VisualizationSoCBProxy&) = delete;
    VisualizationSoCBProxy& operator=(const VisualizationSoCBProxy&) = delete;

    VisualizationSoCBProxy(VisualizationSoCBProxy&&) = default;
    VisualizationSoCBProxy& operator=(VisualizationSoCBProxy&&) = default;
    VisualizationSoCBProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          CameraFEH265ImagePDataViz(token.GetCameraFEH265ImagePDataViz()),
          LidarInno3PPC2Viz(token.GetLidarInno3PPC2Viz()),
          LidarInno4PPC2Viz(token.GetLidarInno4PPC2Viz()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<VisualizationSoCBProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::CameraFEH265ImagePDataViz CameraFEH265ImagePDataViz;
    events::LidarInno3PPC2Viz LidarInno3PPC2Viz;
    events::LidarInno4PPC2Viz LidarInno4PPC2Viz;
};
} // namespace proxy
} // namespace visualization
} // namespace ap
} // namespace jac

#endif // JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_PROXY_H
